@@ -36,12 +36,19 @@ class NotebookWrapper:
         return self.run(*args, *kwargs)
 
     def run(self, *args, **kwargs) -> Any | List[Any]:
+        return self.export(None, *args, **kwargs)
+
+    def export(self, _outputNotebook: str | Path | None, *args, **kwargs):
+        if isinstance(_outputNotebook, str):
+            _outputNotebook = Path(_outputNotebook)
+
         # map input
         variableMapping = dict(zip(self.inputVariable, args))
         variableMapping.update(kwargs)
 
         nb = nbformat.read(self.notebook, as_version=nbformat.NO_CONVERT)
 
+        inputIndex = -1
         if len(variableMapping) > 0:
             # add saving path for input
             inputPath = Path(
@@ -64,30 +71,9 @@ class NotebookWrapper:
             else:
                 raise IOError(inputPath.__str__() + " took too much time to write.")
 
-            # identify input cell
-            inputIndex = 0
-            for i, cell in enumerate(nb.cells):
-                if "tags" in cell.metadata and self.inputTag in cell.metadata["tags"]:
-                    inputIndex = i
-                    break
-                pass
-
-            # add input cell to nb
-            newCell = nbbase.new_code_cell(
-                source="""
-                from pathlib import Path
-                import pickle
-                
-                inputVariables = pickle.loads(Path("%s").read_bytes())
-                for key, value in inputVariables.items():
-                    globals()[key] = value
-                    pass
-            """
-                % inputPath
-            )
-
-            nb.cells.insert(inputIndex + 1, newCell)
+            inputIndex = self._insertInputCell(inputPath)
 
+        outputIndex = -1
         if self.outputVariable is not None:
             # add saving path for output
             outputPath = Path(
@@ -99,29 +85,35 @@ class NotebookWrapper:
             )
             outputPath.parent.mkdir(parents=True, exist_ok=True)
 
-            # add output cell to nb
-            if isinstance(self.outputVariable, List):
-                requestVars = "[" + ",".join(self.outputVariable) + "]"
-            else:
-                requestVars = self.outputVariable
-
-            newCell = nbbase.new_code_cell(
-                source="""
-                from pathlib import Path
-                import pickle
-                
-                outputVariable = %s
-                Path("%s").write_bytes(pickle.dumps(outputVariable))
-            """
-                % (requestVars, outputPath)
-            )
-
-            nb.cells.append(newCell)
+            outputIndex = self._insertOutputCell(outputPath)
             pass
 
         ep = ExecutePreprocessor(timeout=None)
         resultNb, _ = ep.preprocess(nb, {"metadata": {"path": self.notebook.parent}})
 
+        if _outputNotebook is not None:
+            if inputIndex >= 0:
+                resultNb.cells.pop(inputIndex)
+
+                # Add markdown cell noting injected variables
+                mdText = "`functionize-notebook` has modified this notebook during execution. The following variables have been injected:\n\n"
+                for variable, varValue in variableMapping.items():
+                    try:
+                        varStr = str(varValue)
+                    except Exception:
+                        varStr = "This variable could not be represent in text."
+                    mdText += f"- {variable}: {varStr}\n"
+                
+                mdCell = nbbase.new_markdown_cell(source=mdText)
+                resultNb.cells.insert(inputIndex, mdCell)
+                
+            if outputIndex >= 0:
+                resultNb.cells.pop(outputIndex)
+
+            with open(_outputNotebook, "w") as f:
+                nbformat.write(resultNb, f)
+                pass
+
         if self.outputVariable is not None:
             # wait for nb output
             for _ in range(50):
@@ -137,3 +129,56 @@ class NotebookWrapper:
             return res
         else:
             return None
+
+    def _insertInputCell(self, inputPath: Path):
+        nb = nbformat.read(self.notebook, as_version=nbformat.NO_CONVERT)
+
+        # identify input cell
+        inputIndex = 0
+        for i, cell in enumerate(nb.cells):
+            if "tags" in cell.metadata and self.inputTag in cell.metadata["tags"]:
+                inputIndex = i
+                break
+            pass
+
+        newCell = nbbase.new_code_cell(
+            source="""
+                from pathlib import Path
+                import pickle
+                
+                inputVariables = pickle.loads(Path("%s").read_bytes())
+                for key, value in inputVariables.items():
+                    globals()[key] = value
+                    pass
+            """
+            % inputPath
+        )
+
+        nb.cells.insert(inputIndex + 1, newCell)
+
+        return inputIndex + 1
+
+    def _insertOutputCell(
+        self,
+        outputPath: Path,
+    ):
+        nb = nbformat.read(self.notebook, as_version=nbformat.NO_CONVERT)
+
+        if isinstance(self.outputVariable, List):
+            requestVars = "[" + ",".join(self.outputVariable) + "]"
+        else:
+            requestVars = self.outputVariable
+        newCell = nbbase.new_code_cell(
+            source="""
+                from pathlib import Path
+                import pickle
+                
+                outputVariable = %s
+                Path("%s").write_bytes(pickle.dumps(outputVariable))
+            """
+            % (requestVars, outputPath)
+        )
+
+        nb.cells.append(newCell)
+
+        return len(nb.cells) - 1
